`gulp-offline`是基于gulp的离线包快速打包工具，可以将在线资源快速打包成为符合离线包标准的包文件（zip文件）。

## 使用方法
1. 使用`npm install gulp-offline`指令安装`gulp-offline`包。
2. 复制一份config/map-sample.js文件，并重命名为config/map.js。
3. 按map-sample.js中的注释或下文的介绍编辑map.js。
4. 编辑完成以后，在工作目录中执行`gulp offline`，`gulp-offline`就会开始打包工作。
5. 打包完成以后，如果有出现错误，在console末尾部分会有ERROR级别的错误日志输出。亦可以在offline.log文件中找到日志记录。
6. 离线包文件会保存在build目录中，以`module`命名的文件夹内。

## 配置文件
配置文件是一个nodejs模块，定义了一个命名为map的object。
```js
// 离线包配置文件，定义离线包配置信息
this.map = {
	// 模块名称。可以通过模块名称在日志文件中方便查找信息，或者使用-p参数+模块名称，让构建工具只打包指定的离线包。
	// 模块的值可以是一个数组，数组中每个成员都是一个字符串类型的URL地址，用于指定离线包下载的文件地址。
	"module.name1" : [
		"http://www.nodejs.org/node.js"
	],
	// 模块的值也可以是一个object对象，其中包括三个属性。
	"module.name2" : {
        // boolean，可选参数，默认值为true。
        // analyzeStyles设置为true以后，构建工具在下载当前模块中的css文件时，会自动尝试分析文件中引用的资源并一起下载打包。
        analyzeStyles: true,
        // object，可选参数。
        // replacements指定以后，可以以key-value的形式指定下载时需要替换的文本信息。
        // 如本sample中，构建工具在下载完成http://www.nodejs.org/node.js以后，会在文件内全局搜索包含有“alert("hello world!");”内容的字符串，并替换为“alert("hello tencent!");”。
		replacements: {
			"http://www.nodejs.org/node.js": [
				{
					original: 'alert("hello world!");',
					replacement: 'alert("hello tencent!");'
				}
			]
		},
        // files的值是一个数组，数组中每个成员都是一个字符串类型的URL地址，用于指定离线包下载的文件地址。
		files:[
			"http://www.nodejs.org/node.js"
		]
	}
};
```

## 注意
1. 执行离线包任务时，如果`gulp-offline`下载某（几）个文件失败时，只要有一个文件是成功的，离线包文件仍能正常生成，但是离线包文件中将会**只包含**下载成功的文件。
2. 如果您在map.js中定义的模块过多时，执行`gulp offline`指令会运行**全部**的任务，这可能会花费额外的时间。如果您只希望运行其中的某个任务，可以使用`-p`参数。如：`gulp offline -p module.name1`。