var	Thenjs = require('thenjs'),
	gulp = require('gulp'),
	del = require('del'),
	yargs = require('yargs'),
	log4js = require('log4js'),
    zip = require('gulp-zip'),
    replace = require('gulp-replace'),
    url = require('url'),
	download = require('./download'),
    styleAnalyzer = require('./style-analyzer'),
	util = require('./util');

// 离线包构建指令
gulp.task('offline', function(cb) {
	// 配置日志组件
	log4js.configure({
		appenders: [
			{
				type: 'console'
			},
			{
				type: 'file',
				filename: 'offline.log',
				maxLogSize: 1024 * 1024 * 1024,
				backups: 3
			}
		]
	});

	var logger = log4js.getLogger('offline');

    // 获取命令行可选-p 参数
	var argv = yargs.option('p',{
		alias : 'project',
		demand : true,
		default : 'all',
		describe : 'p specifies the project you want to offline-pack',
		type : 'string'
	}).argv;

	// 获取离线包配置项目列表
	var configs = require('./config/map.js') || {};
	configs.keymap = Object.keys(configs.map) || [];

	// 检查-p参数指定项目是否存在
	var checkPass = true;
	if(argv.p != 'all'){
		var _p = argv.p.split(',') || [];
		_p.forEach(function (key) {
			if(!configs.map[key]){
				checkPass = false;
				logger.error(key + ' connot find in config/map.js');
			}
		});
		//没有通过-p参数校验，不执行
		if(!checkPass){
			return;
		}
		configs.keymap = _p;
	}

	// 错误队列
	var errorList = [];

	// 按配置进行串行遍历
	Thenjs.eachSeries(configs.keymap, function(tasksCont, configName){
		// 读取配置信息
		var files, replacements,
			analyzeStyles = true;

		// 读取配置信息
		var config = configs.map[configName];

		// 判断配置信息是否是数组，如果是就直接把数组中的信息读取到files中，否则读取配置信息中的files属性成员
		if(config.constructor === Array){
			files = config.slice();
		}else{
			if(config.files && config.files.constructor === Array){
				files = config.files.slice();
			}

			if(config.replacements){
				replacements = util.cloneObject(config.replacements);
			}

			if(typeof config.analyzeStyles == 'boolean'){
				analyzeStyles = config.analyzeStyles;
			}
		}

		// 如果没有读取到文件信息
		if(!(files && files.length > 0)){
			logger.warn(configName + ' no file(s) found in map.js, this task will be skipped.');
			tasksCont();
			return;
		}

		var tempPrefix = "temp/" + configName + "/"
		var buildPrefix = "build/" + configName + "/"

		logger.info(configName + ' cleaning temporary and target directories and files.');
		// 清理offline环境
		del([tempPrefix, buildPrefix], function(err, path){
			if(err){
				errorList.push({
					error: err
				});
				tasksCont();
				return;
			}
			logger.info(configName + ' clean completed.');

			// 任务队列
			var index = 0;
			var pendingTasks = files.slice();
			var tasks;
			var failedCount = 0;
			var successCount = 0;

			var taskProcessor = function(tasks){
				logger.info(configName + ' ' + tasks.length + ' file(s) will be downloaded.');
				Thenjs.eachSeries(tasks, function(fileCont, file){
					var dirPath = file.replace("http://", "");
					dirPath = dirPath.substr(0, dirPath.lastIndexOf('/')+1);

					var onDownloadCompleted = function(error){
						++index;
						if(error){
							errorList.push({
								url: file,
								error: error
							});
							++failedCount;
							logger.error(configName + ' ' + index + ' of ' + tasks.length + ' download failed.\n' + file);
						}else{
							++successCount;
							logger.info(configName + ' ' + index + ' of ' + tasks.length + ' download completed.');
						}

						if(index == tasks.length) {
							if(pendingTasks.length > 0){
								logger.info(configName + ' begin to download analytical style files, total: '+pendingTasks.length+' file(s).');
								tasks = pendingTasks.slice();
								pendingTasks = [];
								index = 0;
								taskProcessor(tasks);
							}else{
								if(successCount === 0){
									logger.error(configName + ' all file(s) download failed, the task of ' + configName + ' totally failed.');
								}else if(failedCount>0){
									logger.warn(configName + ' ' + successCount + ' file(s) download succeeded, '+ failedCount + ' file(s) download failed.');
								}else{
									logger.info(configName + ' all file(s) download completed.');
								}

								if(successCount > 0){
									gulp.src(tempPrefix+ '**/*')
										.pipe(zip('offline.zip'))
										.pipe(gulp.dest(buildPrefix));
									logger.info(configName + ' zip file has been generated.');
								}

								index = 0;
								failedCount = 0;
								successCount = 0;
								tasksCont();
							}
						}
						fileCont();
					}

					var stream = download(file, onDownloadCompleted);

					if(analyzeStyles && /.*\.css$/.test(file)){
						stream = stream.pipe(styleAnalyzer(function(identifiedFiles){
							var filesObj = {};
							// 用object来当hashSet
							for(var i=0;i<tasks;++i){
								filesObj[tasks[i]] = '';
							}
							// 向files添加解析出的文件
							for(var i=0;i<identifiedFiles.length;++i){
								var fileUrl = url.resolve(file, identifiedFiles[i]);
								var fileUrlObj = url.parse(fileUrl);
								fileUrlObj.search = '';
								fileUrlObj.hash = '';
								fileUrl = url.format(fileUrlObj);

								if(!filesObj.hasOwnProperty[fileUrl]){
									// 将分析出来的文件保存在analyticalFiles中
									pendingTasks.push(fileUrl);
								}
							}
						}));
					}

					if(replacements &&
						replacements[file]) {
						var replacementConfig = replacements[file];
						for (var i = 0; i < replacementConfig.length; ++i) {
							stream = stream.pipe(replace(replacementConfig[i].original, replacementConfig[i].replacement));
						}
					}

					if(!stream){
						logger.error(configName + ' cannot download file.\n' + file);
						return;
					}

					stream = stream.pipe(gulp.dest(tempPrefix + dirPath));

					stream.on('end', onDownloadCompleted);
				});
			};

			tasks = pendingTasks.slice();
			pendingTasks = [];
			index = 0;
			taskProcessor(tasks);
		});
	}).fin(function(cont, error, result){
		if(errorList.length > 0){
			logger.error(result.length + ' task(s) completed, but not all successfully. here is the report of failed file(s).');
			logger.error('================================================================================');
			for(var i= 0,e;e=errorList[i];++i){
				if(typeof e.error == 'object'){
					logger.error(e.url + ' failed reason:\n' + JSON.stringify(e.error));
				}
				if(typeof e.error == 'number'){
					logger.error(e.url + ' failed because of http status code: ' + e.error);
				}
			}
		}else{
			logger.info(result.length + ' task(s) all finished.');
		}
		cont();
		cb();
	});
});